import { FileImageOutlined } from "@ant-design/icons";
import { Col, Image, Row, Tag } from "antd";
import MyStlViewer from "../MyStlViewer";

/**
 * Attachments is an array of objects with the following structure:
 *
 * [{OriginalFileName: "test.png", SystemFileName: "7344ac51-6dc6-4a6a-94ae-6e7e4dd2e823.txt"}]
 *
 * Supports images, files and stl files.
 */
export default function MyAttachments({ attachments, downloadUrl }) {
  const getDownloadUrl = (file) => {
    return `${downloadUrl}${file.SystemFileName}`;
  };

  const generateTagWithDownloadLink = (file) => (
    <a
      key={"a" + file.SystemFileName}
      href={getDownloadUrl(file)}
      download="test"
      target="_blank"
      rel="noreferrer"
    >
      <Tag
        icon={<FileImageOutlined />}
        color="processing"
        style={{ marginTop: 6 }}
      >
        {file.OriginalFileName}
      </Tag>
    </a>
  );

  const imageFiles = attachments.filter((file) => {
    const fileExtension = file.SystemFileName.split(".")[1];
    return ["png", "jpeg", "jpg", "webp"].includes(fileExtension);
  });

  const nonImageFiles = attachments.filter((file) => {
    const fileExtension = file.SystemFileName.split(".")[1];
    return !["png", "jpeg", "jpg", "webp"].includes(fileExtension);
  });

  const nonImageElements = nonImageFiles.map((nonImageFile) => {
    const fileExtension = nonImageFile.SystemFileName.split(".")[1];

    if (fileExtension === "stl") {
      return (
        <div key={"tag" + nonImageFile.SystemFileName}>
          <div style={{ height: 300, backgroundColor: "#fff" }}>
            <MyStlViewer url={getDownloadUrl(nonImageFile)} />
          </div>
          {generateTagWithDownloadLink(nonImageFile)}
        </div>
      );
    } else {
      return generateTagWithDownloadLink(nonImageFile);
    }
  });

  const imageFileElements = imageFiles.map((imageFile) => (
    <Col xs={24} sm={12} lg={6} key={imageFile.SystemFileName}>
      <Image width={"100%"} src={getDownloadUrl(imageFile)} />
      {generateTagWithDownloadLink(imageFile)}
    </Col>
  ));

  const imageRow = imageFileElements.length > 0 && (
    <Row key={"imgrow"} gutter={{ xs: 8, sm: 16, md: 24, lg: 32 }}>
      {imageFileElements}
    </Row>
  );

  return [...nonImageElements, imageRow];
}
