import { createContext, useContext, useRef, useState } from "react";

const preview = {
  paginationPage: 1,
  paginationPageRef: null,
  totalPages: 0,
  customers: [],
  openDrawerCustomerId: null,
  currentDrawerCustomer: null,
  currentDrawerCustomerRef: null,
  currentDrawerCustomerNotesRef: null,
  changedDrawerCustomerFieldsRef: null,
};

const CrmContext = createContext(preview);

export const useCrmContext = () => useContext(CrmContext);

export function CrmProvider({ children }) {
  const [paginationPage, setPaginationPage] = useState(1);
  const paginationPageRef = useRef(paginationPage);
  const [totalPages, setTotalPages] = useState(0);

  const [customers, setCustomers] = useState([]);
  // will be used to store the customer id that is currently being viewed in the drawer
  const openDrawerCustomerId = useRef(null);
  // will be used to store the customer object that is currently being viewed in the drawer
  const [currentDrawerCustomer, setCurrentDrawerCustomer] = useState(null);
  // will be used to store the customer updated object that is currently being viewed in the drawer
  const currentDrawerCustomerRef = useRef(null);
  // this will be used to store the updates noted and will be compared to currentDrawerCustomerRef to see if there are any changes
  const currentDrawerCustomerNotesRef = useRef(null);
  const changedDrawerCustomerFieldsRef = useRef([]);

  return (
    <CrmContext.Provider
      value={{
        paginationPage,
        setPaginationPage,
        paginationPageRef,
        totalPages,
        setTotalPages,
        customers,
        setCustomers,
        openDrawerCustomerId,
        currentDrawerCustomer,
        setCurrentDrawerCustomer,
        currentDrawerCustomerNotesRef,
        currentDrawerCustomerRef,
        changedDrawerCustomerFieldsRef,
      }}
    >
      {children}
    </CrmContext.Provider>
  );
}
