import { createContext, useContext, useRef, useState } from "react";

const preview = {
  checkboxAutomaticFillingOfInputFieldsChecked: false,
  setCheckboxAutomaticFillingOfInputFieldsChecked: () => {},
  checkboxAutomaticFillingOfInputFieldsCheckedRef: null,

  checkboxAutomaticRecognitionOfEquipmentDocumentationChecked: false,
  setCheckboxAutomaticRecognitionOfEquipmentDocumentationChecked: () => {},
  checkboxAutomaticRecognitionOfEquipmentDocumentationCheckedRef: null,
};

const ScannerContext = createContext(preview);

export const useScannerContext = () => useContext(ScannerContext);

export default function ScannerProvider({ children }) {
  // automatic filling of input fields according to scan result
  const lCheckboxAutomaticFillingOfInputFieldsChecked =
    localStorage.getItem("cAFOIF") === "true";

  const [
    checkboxAutomaticFillingOfInputFieldsChecked,
    setCheckboxAutomaticFillingOfInputFieldsChecked,
  ] = useState(lCheckboxAutomaticFillingOfInputFieldsChecked);
  const checkboxAutomaticFillingOfInputFieldsCheckedRef = useRef(
    lCheckboxAutomaticFillingOfInputFieldsChecked
  );

  const saveCheckboxAutomaticFillingOfInputFieldsChecked = (checked) => {
    localStorage.setItem("cAFOIF", checked);

    setCheckboxAutomaticFillingOfInputFieldsChecked(checked);
    checkboxAutomaticFillingOfInputFieldsCheckedRef.current = checked;
  };

  // automatic detection of equipment documentation
  const lCheckboxAutomaticRecognitionOfEquipmentDocumentationChecked =
    localStorage.getItem("cAROED") === "true";

  const [
    checkboxAutomaticRecognitionOfEquipmentDocumentationChecked,
    setCheckboxAutomaticRecognitionOfEquipmentDocumentationChecked,
  ] = useState(lCheckboxAutomaticRecognitionOfEquipmentDocumentationChecked);
  const checkboxAutomaticRecognitionOfEquipmentDocumentationCheckedRef = useRef(
    lCheckboxAutomaticRecognitionOfEquipmentDocumentationChecked
  );

  const saveCheckboxAutomaticRecognitionOfEquipmentDocumentationChecked = (
    checked
  ) => {
    localStorage.setItem("cAROED", checked);

    setCheckboxAutomaticRecognitionOfEquipmentDocumentationChecked(checked);
    checkboxAutomaticRecognitionOfEquipmentDocumentationCheckedRef.current =
      checked;
  };

  return (
    <ScannerContext.Provider
      value={{
        checkboxAutomaticFillingOfInputFieldsChecked,
        setCheckboxAutomaticFillingOfInputFieldsChecked:
          saveCheckboxAutomaticFillingOfInputFieldsChecked,
        checkboxAutomaticFillingOfInputFieldsCheckedRef,
        checkboxAutomaticRecognitionOfEquipmentDocumentationChecked,
        setCheckboxAutomaticRecognitionOfEquipmentDocumentationChecked:
          saveCheckboxAutomaticRecognitionOfEquipmentDocumentationChecked,
        checkboxAutomaticRecognitionOfEquipmentDocumentationCheckedRef,
      }}
    >
      {children}
    </ScannerContext.Provider>
  );
}
