import { Form, Input, Modal } from "antd";
import { useState } from "react";
import { Constants } from "../../../utils";
import { SentMessagesCommands } from "../../../Handlers/WebSocketMessageHandler";

export default function AddLogManagerServerConnectionModal({
  isModalOpen,
  setIsModalOpen,
  t,
  webSocketContext,
}) {
  const [displayName, setDisplayName] = useState("");
  const [address, setAddress] = useState("");

  const isAddLogManagerServerConnectionPossible = () => {
    if (
      displayName.length <
        Constants.GLOBALS.MIN_LOG_MANAGER_DISPLAY_NAME_LENGTH ||
      displayName.length >
        Constants.GLOBALS.MAX_LOG_MANAGER_DISPLAY_NAME_LENGTH ||
      address.length < Constants.GLOBALS.MIN_LOG_MANAGER_ADDRESS_LENGTH ||
      address.length > Constants.GLOBALS.MAX_LOG_MANAGER_ADDRESS_LENGTH
    )
      return false;

    return true;
  };

  const onConfirmAddLogManagerServerConnection = () => {
    webSocketContext.SendSocketMessage(
      SentMessagesCommands.AdminAreaManageAddLogManagerServerConnection,
      {
        DisplayName: displayName,
        Address: address,
      }
    );

    setDisplayName("");
    setAddress("");

    setIsModalOpen(false);
  };

  return (
    <Modal
      title={t(
        "adminArea.manage.logManagers.addLogManagerServerConnectionModal.title"
      )}
      open={isModalOpen}
      centered
      maskClosable={false}
      okText={t("common.button.create")}
      cancelText={t("common.button.cancel")}
      okButtonProps={{ disabled: !isAddLogManagerServerConnectionPossible() }}
      onCancel={() => setIsModalOpen(false)}
      onOk={() => onConfirmAddLogManagerServerConnection()}
    >
      <Form layout="vertical">
        <Form.Item
          label={t(
            "adminArea.manage.logManagers.addLogManagerServerConnectionModal.displayName"
          )}
          hasFeedback
          validateStatus={
            displayName.length !== 0 &&
            displayName.length <
              Constants.GLOBALS.MIN_LOG_MANAGER_DISPLAY_NAME_LENGTH &&
            "error"
          }
        >
          <Input
            placeholder={t(
              "adminArea.manage.logManagers.addLogManagerServerConnectionModal.displayName"
            )}
            value={displayName}
            onChange={(e) => setDisplayName(e.target.value)}
            minLength={Constants.GLOBALS.MIN_LOG_MANAGER_DISPLAY_NAME_LENGTH}
            maxLength={Constants.GLOBALS.MAX_LOG_MANAGER_DISPLAY_NAME_LENGTH}
          />
        </Form.Item>

        <Form.Item
          label={t(
            "adminArea.manage.logManagers.addLogManagerServerConnectionModal.address"
          )}
          hasFeedback
          validateStatus={
            address.length !== 0 &&
            address.length < Constants.GLOBALS.MIN_LOG_MANAGER_ADDRESS_LENGTH &&
            "error"
          }
        >
          <Input
            placeholder={"http://127.0.0.1:50110/v1/log"}
            type="url"
            value={address}
            onChange={(e) => setAddress(e.target.value)}
            minLength={Constants.GLOBALS.MIN_LOG_MANAGER_ADDRESS_LENGTH}
            maxLength={Constants.GLOBALS.MAX_LOG_MANAGER_ADDRESS_LENGTH}
          />
        </Form.Item>
      </Form>
    </Modal>
  );
}
