import { Col, Result, Row, Select, Space } from "antd";
import LogCard from "../../Components/LogCard";
import { useEffect, useState } from "react";
import { AppStyle, myFetch, wsConnectionCustomEventName } from "../../utils";
import { useConsolesContext } from "../../Contexts/ConsolesContext";
import { useTranslation } from "react-i18next";

export default function Consoles() {
  const { t } = useTranslation();
  const consolesContext = useConsolesContext();

  const [selectedLogServer, setSelectedLogServer] = useState("");
  const [availableLogTypes, setAvailableLogTypes] = useState([]);
  const [selectedLogType, setSelectedLogType] = useState("");
  const [result, setResult] = useState();

  useEffect(() => {
    const lmscRequest = () =>
      myFetch("/lmsc", "GET").then((data) => {
        consolesContext.setConnectedLogManagerServers(data);
        setSelectedLogServer(data.length > 0 ? data[0].Address : "");
      });

    lmscRequest();

    const handleWsConnectionEvent = () => lmscRequest();

    document.addEventListener(
      wsConnectionCustomEventName,
      handleWsConnectionEvent
    );

    return () =>
      document.removeEventListener(
        wsConnectionCustomEventName,
        handleWsConnectionEvent
      );
  }, []);

  useEffect(() => {
    if (selectedLogServer === "") return;

    myFetch("/types", "GET", null, {}, 0, selectedLogServer)
      .then((data) => {
        let newTypes = [];

        if (data.length > 0) {
          newTypes = data.map((type) => {
            return { label: type, value: type };
          });

          setResult();
        } else {
          setResult(
            <Result
              status="404"
              title={t("consoles.noLogTypesFound")}
              subTitle={t("common.contactAdmin")}
            />
          );
        }

        setAvailableLogTypes(newTypes);
        setSelectedLogType(newTypes.length > 0 ? newTypes[0].value : "");
      })
      .catch(() => {
        setResult(
          <Result
            status="500"
            title={t("consoles.couldntReachlogManagerServer")}
            subTitle={t("common.contactAdmin")}
          />
        );
      });
  }, [selectedLogServer]);

  return (
    <LogCard
      type={selectedLogType}
      logApiAddress={selectedLogServer}
      customResult={
        consolesContext.connectedLogManagerServers.length > 0 ? (
          result
        ) : (
          <Result
            status="500"
            title={t("consoles.noLogManagerServerSpecified")}
            subTitle={t("common.contactAdmin")}
          />
        )
      }
      title={
        <Row gutter={[16, 16]}>
          <Col xs={24} sm={6}>
            <Select
              disabled={consolesContext.connectedLogManagerServers.length === 0}
              style={{ width: "100%", marginRight: AppStyle.app.margin }}
              options={consolesContext.connectedLogManagerServers.map(
                (connection) => {
                  return {
                    label: connection.DisplayName,
                    value: connection.Address,
                  };
                }
              )}
              value={selectedLogServer}
              onChange={(value) => setSelectedLogServer(value)}
            />
          </Col>

          <Col xs={24} sm={18}>
            <Select
              disabled={
                result !== undefined ||
                consolesContext.connectedLogManagerServers.length === 0
              }
              style={{ width: "100%" }}
              options={availableLogTypes}
              value={selectedLogType}
              onChange={(value) => setSelectedLogType(value)}
            />
          </Col>
        </Row>
      }
    />
  );
}
