import {
  ArrowDownOutlined,
  ArrowUpOutlined,
  CameraOutlined,
  DeleteOutlined,
  EllipsisOutlined,
  FullscreenOutlined,
  PlusOutlined,
  SearchOutlined,
} from "@ant-design/icons";
import {
  Button,
  Card,
  Col,
  Dropdown,
  Input,
  Result,
  Row,
  Select,
  Space,
  Typography,
  Upload,
  message,
} from "antd";
import { AppStyle, Constants, myFetch } from "../../utils";
import { useEffect, useRef, useState } from "react";
import MyModal, {
  MyModalCloseCreateButtonFooter,
  MyModalCloseSaveButtonFooter,
} from "../../Components/MyModal";
import Webcam from "react-webcam";
import TextArea from "antd/es/input/TextArea";
import { useTranslation } from "react-i18next";
import MyTypography from "../../Components/MyTypography";
import { useNavigate } from "react-router-dom";
import MyQrScanner from "../../Components/MyQrScanner";

export default function EquipmentDocumentationOverview() {
  const { t } = useTranslation();

  return (
    <>
      <ScannerComponent />

      <Result
        status="404"
        title={t(
          "equipmentDocumentationOverview.noEquipmentScannedResult.title"
        )}
        subTitle={t(
          "equipmentDocumentationOverview.noEquipmentScannedResult.description"
        )}
      />
    </>
  );
}

export const IsScannedQrCodeValid = (result) => {
  // {"stockitem": 11} or 11 is valid

  try {
    // Try to parse the scanned content as JSON
    const parsedResult = JSON.parse(result);

    // Check if the parsed object has the expected format
    if (parsedResult && typeof parsedResult === "object") {
      if (
        parsedResult.stockitem !== undefined &&
        typeof parsedResult.stockitem === "number"
      ) {
        return true; // The QR code has the expected format
      }
    } else if (!isNaN(parsedResult)) {
      return true; // The QR code is a valid number
    }

    return false; // The QR code does not have the expected format or value
  } catch (error) {
    return false; // Error while parsing the QR code
  }
};

export function OpenScannedEquipmentDocumentation({
  navigate,
  scannedEquipment,
}) {
  navigate(
    `${Constants.ROUTE_PATHS.EQUIPMENT_DOCUMENTATION_VIEW}${scannedEquipment}`
  );
}

export function ScannerComponent() {
  const navigate = useNavigate();
  const { t } = useTranslation();

  const [messageApi, messageContextHolder] = message.useMessage({
    maxCount: 2,
  });

  const [isScannerActive, setIsScannerActive] = useState(false);
  const [inputValue, setInputValue] = useState("");

  return (
    <>
      {messageContextHolder}

      <MyQrScanner
        isScannerActive={isScannerActive}
        setIsScannerActive={setIsScannerActive}
        previewTitle={t("equipmentDocumentationOverview.scanEquipment.title")}
        onDecode={(result) => {
          if (!IsScannedQrCodeValid(result)) {
            messageApi.error(
              t("equipmentDocumentationOverview.messageErrorInvalidStockItem")
            );
            return;
          }

          setIsScannerActive(false);

          OpenScannedEquipmentDocumentation({
            navigate: navigate,
            scannedEquipment: JSON.parse(result).stockitem.toString(),
          });
        }}
        onError={(error) => console.error(error?.message)}
        bottomContent={
          <Row gutter={AppStyle.grid.row.gutter}>
            <Col xs={24} xl={16}>
              <Input
                placeholder={t(
                  "equipmentDocumentationOverview.scanEquipment.inputPlaceholder"
                )}
                value={inputValue}
                onInput={(e) => setInputValue(e.target.value)}
              />
            </Col>
            <Col xs={24} xl={8}>
              <Button
                block
                icon={<SearchOutlined />}
                onClick={() => {
                  if (!IsScannedQrCodeValid(inputValue)) {
                    messageApi.error(
                      t(
                        "equipmentDocumentationOverview.messageErrorInvalidStockItem"
                      )
                    );
                    return;
                  }

                  OpenScannedEquipmentDocumentation({
                    navigate: navigate,
                    scannedEquipment: inputValue,
                  });
                }}
              >
                {t("equipmentDocumentationOverview.scanEquipment.buttonSearch")}
              </Button>
            </Col>
          </Row>
        }
      />
    </>
  );
}

/*
<Row style={{ marginBottom: AppStyle.app.margin }}>
        <Col xs={0} md={7} />
        <Col xs={24} md={10}>
          <Card
            style={{
              textAlign: "center",
            }}
          >
            {isScannerActive ? (
              <>
                <QrScanner
                  onDecode={(result) => {
                    if (!isScannedQrCodeValid(result)) {
                      messageApi.error(
                        t(
                          "equipmentDocumentationOverview.messageErrorInvalidStockItem"
                        )
                      );
                      return;
                    }

                    setIsScannerActive(false);

                    handleScannedEquipment(
                      JSON.parse(result).stockitem.toString()
                    );
                  }}
                  onError={(error) => console.error(error?.message)}
                />
                <Button
                  block
                  onClick={() => setIsScannerActive(false)}
                  style={{
                    marginTop: AppStyle.app.margin,
                    marginBottom: AppStyle.app.margin,
                  }}
                >
                  {t("equipmentDocumentationOverview.buttonCloseCamera")}
                </Button>
              </>
            ) : (
              <div onClick={() => setIsScannerActive(true)}>
                <CameraOutlined style={{ fontSize: 64 }} />
                <Typography.Title level={5}>
                  {t("equipmentDocumentationOverview.scanEquipment.title")}
                </Typography.Title>
              </div>
            )}

            <Row gutter={AppStyle.grid.row.gutter}>
              <Col xs={24} xl={16}>
                <Input
                  placeholder={t(
                    "equipmentDocumentationOverview.scanEquipment.inputPlaceholder"
                  )}
                  value={inputValue}
                  onInput={(e) => setInputValue(e.target.value)}
                />
              </Col>
              <Col xs={24} xl={8}>
                <Button
                  block
                  icon={<SearchOutlined />}
                  onClick={() => {
                    if (!isScannedQrCodeValid(inputValue)) {
                      messageApi.error(
                        t(
                          "equipmentDocumentationOverview.messageErrorInvalidStockItem"
                        )
                      );
                      return;
                    }

                    handleScannedEquipment(inputValue);
                  }}
                >
                  {t(
                    "equipmentDocumentationOverview.scanEquipment.buttonSearch"
                  )}
                </Button>
              </Col>
            </Row>
          </Card>
        </Col>
        <Col xs={0} md={7} />
      </Row>
*/

export const EmptyNote = { Image: null, Description: "" };

export function EquipmentDocumentationViewEditComponent({
  createMode,
  isOpen,
  fetchDocumentation,
  stockItemId,
  documentationId,
}) {
  const navigate = useNavigate();
  const { t } = useTranslation();
  const [title, setTitle] = useState(
    createMode
      ? t("equipmentDocumentationViewEditComponent.titleNewDocumentation")
      : ""
  );
  const [selectedDocumentationType, setSelectedDocumentationType] = useState(
    t(
      "equipmentDocumentationViewEditComponent.selectDocumentationTypeOptions",
      { returnObjects: true }
    )[0].value
  );
  const [notes, setNotes] = useState([EmptyNote]);
  const [isDocumentationUploading, setIsDocumentationUploading] =
    useState(false);
  const documentationResponse = useRef();

  const handleCancel = () => {
    navigate(
      `${Constants.ROUTE_PATHS.EQUIPMENT_DOCUMENTATION_VIEW}${stockItemId}`
    );
  };

  const handleCreate = () => {
    setIsDocumentationUploading(true);

    const updatedNotes = [...notes];

    updatedNotes.forEach((note, index) => {
      if (note.Image === null && note.Description === "") {
        updatedNotes.splice(index, 1);
      }
    });

    let body = {
      stockItemId: stockItemId,
      type: selectedDocumentationType,
      title: title,
      notes: updatedNotes,
    };

    myFetch(`/equipment/documentation/create`, "POST", body, {}).then(() => {
      setIsDocumentationUploading(false);
      fetchDocumentation();
      handleCancel();
    });

    navigate(
      `${Constants.ROUTE_PATHS.EQUIPMENT_DOCUMENTATION_VIEW}${stockItemId}`
    );
  };

  const handleEdit = () => {
    setIsDocumentationUploading(true);

    const updatedNotes = [...notes];

    updatedNotes.forEach((note, index) => {
      if (note.Image?.startsWith("http")) {
        updatedNotes[index].Image = JSON.parse(
          documentationResponse.current.Notes
        )[index].Image;
      }
    });

    let body = {
      documentationId: documentationId,
      type: selectedDocumentationType,
      title: title,
      notes: updatedNotes,
    };

    myFetch(`/equipment/documentation/edit`, "POST", body, {}).then(() => {
      setIsDocumentationUploading(false);
      fetchDocumentation();
      handleCancel();
    });
  };

  const handleDescriptionChange = (index) => (e) => {
    const updatedNotes = [...notes];

    updatedNotes[index] = {
      ...updatedNotes[index],
      Description: e.target.value,
    };

    setNotes(updatedNotes);
  };

  const handleImageChange = (index) => (newImage) => {
    const updatedNotes = [...notes];

    updatedNotes[index] = {
      ...updatedNotes[index],
      Image: newImage,
    };

    setNotes(updatedNotes);
  };

  const handleAddNote = () => setNotes([...notes, EmptyNote]);

  const isAddNoteButtonDisabled = () => {
    const lastNote = notes[notes.length - 1];

    return lastNote.Image === null && lastNote.Description === "";
  };

  const isCreateButtonDisabled = () => {
    if (notes.length === 0) return true;

    if (notes.length === 1) {
      return (
        (notes[0].Image === null && notes[0].Description === "") || title === ""
      );
    }

    return false;
  };

  // triggers on edit documentation
  useEffect(() => {
    if (!isOpen) return;
    if (createMode) return;

    myFetch(
      `/equipment/documentation/${stockItemId}/${documentationId}`,
      "GET"
    ).then((data) => {
      documentationResponse.current = data;

      setTitle(data.Title);

      // adding the static server url in front of the file name
      const updatedDataNotes = JSON.parse(data.Notes);

      updatedDataNotes.forEach((note, index) => {
        if (note.Image !== "") {
          updatedDataNotes[
            index
          ].Image = `${Constants.STATIC_CONTENT_ADDRESS}equipmentdocumentation/${documentationId}/${note.Image}`;
        }
      });

      setNotes(updatedDataNotes);
      setSelectedDocumentationType(data.Type);
    });
  }, [isOpen]);

  return (
    <MyModal
      isOpen={isOpen}
      onCancel={handleCancel}
      title={
        <MyTypography
          value={title}
          setValue={setTitle}
          maxLength={Constants.GLOBALS.MAX_EQUIPMENT_DOCUMENTATION_TITLE_LENGTH}
        />
      }
      footer={
        createMode ? (
          <MyModalCloseCreateButtonFooter
            onCreate={handleCreate}
            onCancel={handleCancel}
            isCreateButtonDisabled={isCreateButtonDisabled()}
            isCreateButtonLoading={isDocumentationUploading}
          />
        ) : (
          <MyModalCloseSaveButtonFooter
            onSave={handleEdit}
            onCancel={handleCancel}
            isSaveButtonLoading={isDocumentationUploading}
          />
        )
      }
    >
      <div style={{ marginBottom: AppStyle.typography.text.marginBottom }}>
        <Typography.Text>
          {t("equipmentDocumentationViewEditComponent.textDocumentationType")}
        </Typography.Text>
      </div>

      <Select
        value={selectedDocumentationType}
        style={{ width: "100%", marginBottom: AppStyle.app.margin }}
        onChange={(value) => setSelectedDocumentationType(value)}
        options={t(
          "equipmentDocumentationViewEditComponent.selectDocumentationTypeOptions",
          { returnObjects: true }
        )}
      />

      {notes.map((note, index) => (
        <NoteComponent
          key={index}
          index={index}
          image={note.Image}
          onImageChange={handleImageChange(index)}
          description={note.Description}
          onDescriptionChange={handleDescriptionChange(index)}
          onDeleteImage={() => handleImageChange(index)(null)}
          documentationId={documentationId}
          onMoveUp={() => {
            if (index === 0) return;

            // updating the notes
            const updatedNotes = [...notes];
            const temp = updatedNotes[index - 1];
            updatedNotes[index - 1] = updatedNotes[index];
            updatedNotes[index] = temp;

            setNotes(updatedNotes);

            if (createMode) return;

            // updating the documentationResponse.current
            const updatedCurrentNotes = JSON.parse(
              documentationResponse.current.Notes
            );

            const temp2 = updatedCurrentNotes[index - 1];
            updatedCurrentNotes[index - 1] = updatedCurrentNotes[index];
            updatedCurrentNotes[index] = temp2;

            documentationResponse.current = {
              ...documentationResponse.current,
              Notes: JSON.stringify(updatedCurrentNotes),
            };
          }}
          onMoveDown={() => {
            if (index === notes.length - 1) return;

            // updating the notes
            const updatedNotes = [...notes];
            const temp = updatedNotes[index + 1];
            updatedNotes[index + 1] = updatedNotes[index];
            updatedNotes[index] = temp;

            setNotes(updatedNotes);

            if (createMode) return;

            // updating the documentationResponse.current
            const updatedCurrentNotes = JSON.parse(
              documentationResponse.current.Notes
            );

            const temp2 = updatedCurrentNotes[index + 1];
            updatedCurrentNotes[index + 1] = updatedCurrentNotes[index];
            updatedCurrentNotes[index] = temp2;

            documentationResponse.current = {
              ...documentationResponse.current,
              Notes: JSON.stringify(updatedCurrentNotes),
            };
          }}
          onDelete={() => {
            if (notes.length === 1) {
              setNotes([EmptyNote]);
              return;
            }

            const updatedNotes = [...notes];
            updatedNotes.splice(index, 1);

            setNotes(updatedNotes);
          }}
        />
      ))}

      <div style={{ textAlign: "center" }}>
        <Button
          type="primary"
          disabled={isAddNoteButtonDisabled()}
          icon={<PlusOutlined />}
          onClick={handleAddNote}
        >
          {t("equipmentDocumentationViewEditComponent.buttonAddNote")}
        </Button>
      </div>
    </MyModal>
  );
}

function UploadComponent({ setImagePreview, messageApi }) {
  const handleBeforeUpload = (file) => {
    if (
      !Constants.ACCEPTED_EQUIPMENT_DOCUMENTATION_FILE_TYPES.includes(file.type)
    ) {
      messageApi.error(`${file.name} is not valid file type`);
      return false;
    }

    const reader = new FileReader();
    reader.onload = () => setImagePreview(reader.result);
    reader.readAsDataURL(file);

    return false;
  };

  return (
    <Upload
      beforeUpload={handleBeforeUpload}
      maxCount={1}
      accept={Constants.ACCEPTED_EQUIPMENT_DOCUMENTATION_FILE_TYPES}
      showUploadList={false}
    >
      <PlusOutlined />
    </Upload>
  );
}

function CameraComponent({ setImagePreview }) {
  const { t } = useTranslation();
  const [modalVisible, setModalVisible] = useState(false);
  const [cameraVisible, setCameraVisible] = useState(false);
  const webcamRef = useRef(null);

  const handleCapture = () => {
    const imageSrc = webcamRef.current.getScreenshot();
    setImagePreview(imageSrc);

    handleCancel();
  };

  const handleCancel = () => {
    setCameraVisible(false);
    // this is needed to close the camera correctly
    setTimeout(() => setModalVisible(false), 100);
  };

  return (
    <>
      <div
        onClick={() => {
          setModalVisible(true);
          setCameraVisible(true);
        }}
      >
        <CameraOutlined />
      </div>

      <MyModal
        isOpen={modalVisible}
        onCancel={handleCancel}
        footer={[
          <Button key={0} block type="primary" onClick={handleCapture}>
            {t("equipmentDocumentationViewEditComponent.buttonTakePicture")}
          </Button>,
        ]}
      >
        {cameraVisible && (
          <Webcam
            audio={false}
            ref={webcamRef}
            screenshotFormat="image/jpeg"
            width={"100%"}
            videoConstraints={{
              width: 1920,
              height: 1080,
              //facingMode: { exact: "environment" },
            }}
          />
        )}
      </MyModal>
    </>
  );
}

function MoreComponent({ onMoveUp, onMoveDown, onDelete }) {
  const { t } = useTranslation();

  return (
    <Dropdown
      trigger={["click"]}
      menu={{
        items: [
          {
            label: (
              <div onClick={onMoveUp}>
                <Space>
                  <ArrowUpOutlined />
                  <span>
                    {t("equipmentDocumentationViewEditComponent.buttonMoveUp")}
                  </span>
                </Space>
              </div>
            ),
            key: "1",
          },
          {
            label: (
              <div onClick={onMoveDown}>
                <Space>
                  <ArrowDownOutlined />
                  <span>
                    {t(
                      "equipmentDocumentationViewEditComponent.buttonMoveDown"
                    )}
                  </span>
                </Space>
              </div>
            ),
            key: "2",
          },
          {
            label: (
              <div onClick={onDelete}>
                <Space>
                  <DeleteOutlined />
                  <span>
                    {t("equipmentDocumentationViewEditComponent.buttonDelete")}
                  </span>
                </Space>
              </div>
            ),
            key: "0",
          },
        ],
      }}
    >
      <EllipsisOutlined />
    </Dropdown>
  );
}

export function NoteComponent({
  viewMode,
  image,
  onImageChange,
  description,
  onDescriptionChange,
  onDeleteImage,
  documentationId,
  onMoveUp,
  onMoveDown,
  onDelete,
}) {
  const { t } = useTranslation();
  const [messageApi, messageContextHolder] = message.useMessage();
  const [isImageFullScreenModalOpen, setIsImageFullScreenModalOpen] =
    useState(false);

  const imageSrc = viewMode
    ? `${Constants.STATIC_CONTENT_ADDRESS}equipmentdocumentation/${documentationId}/${image}`
    : image;

  const FullscreenOutlinedIcon = ({ disabled }) => (
    <FullscreenOutlined
      disabled={true}
      onClick={() => {
        if (disabled) return;

        setIsImageFullScreenModalOpen(true);
      }}
    />
  );

  return (
    <div style={{ marginBottom: AppStyle.app.margin }}>
      {messageContextHolder}

      <Row gutter={[AppStyle.grid.row.gutter[0]]}>
        <Col xs={24} md={8}>
          <Card
            bodyStyle={{ padding: 0 }}
            actions={
              viewMode
                ? image
                  ? [<FullscreenOutlinedIcon />]
                  : null
                : [
                    <UploadComponent
                      setImagePreview={onImageChange}
                      messageApi={messageApi}
                    />,
                    <CameraComponent setImagePreview={onImageChange} />,
                    <DeleteOutlined onClick={onDeleteImage} />,
                    <FullscreenOutlinedIcon
                      disabled={image === null || image === ""}
                    />,
                    <MoreComponent
                      onMoveUp={onMoveUp}
                      onMoveDown={onMoveDown}
                      onDelete={onDelete}
                    />,
                  ]
            }
          >
            {!image ? (
              <div
                style={{
                  height: 250,
                  display: "flex",
                  justifyContent: "center",
                  alignItems: "center",
                }}
              >
                {viewMode
                  ? t(
                      "equipmentDocumentationViewEditComponent.textImageNoImage"
                    )
                  : t(
                      "equipmentDocumentationViewEditComponent.textImageNoImageSelected"
                    )}
              </div>
            ) : (
              <img
                src={imageSrc}
                width="100%"
                style={{
                  borderTopLeftRadius: 6,
                  borderTopRightRadius: 6,
                }}
                alt="Preview"
              />
            )}
          </Card>
        </Col>

        <Col xs={24} md={16}>
          {viewMode ? (
            <Typography.Text>{description}</Typography.Text>
          ) : (
            <TextArea
              autoSize={{ minRows: 8, maxRows: 13 }}
              placeholder={t(
                "equipmentDocumentationViewEditComponent.textareaPlaceholder"
              )}
              value={description}
              onChange={onDescriptionChange}
              showCount
              maxLength={
                Constants.GLOBALS.MAX_EQUIPMENT_DOCUMENTATION_NOTE_LENGTH
              }
            />
          )}
        </Col>
      </Row>

      <ImageFullscreenModal
        isOpen={isImageFullScreenModalOpen}
        onCancel={() => setIsImageFullScreenModalOpen(false)}
        image={imageSrc}
      />
    </div>
  );
}

function ImageFullscreenModal({ isOpen, onCancel, image }) {
  const { t } = useTranslation();

  return (
    <MyModal
      isOpen={isOpen}
      onCancel={onCancel}
      title={t(
        "equipmentDocumentationViewEditComponent.modalImageFullscreenTitle"
      )}
    >
      <img width="100%" src={image} alt="Fullscreen preview" />
    </MyModal>
  );
}
